function DemonstrationMoravec()
%*************************************************************************
%*                                                                       *
%*  Demonstration file of the Moravec operator for corners detection.    *
%*  At first there is necessary to set up parameters of a camera in your *
%*  computer in section "camera setting" below.                          *
%*                                                                       *
%*  Author: Karel Horak, Brno University of Technology, 2012             *
%*                                                                       *
%*************************************************************************

%% cleaning
clear all;
close all;
clc;

%% camera settings
imaqreset;
pause(2);
kam = videoinput('winvideo',1);
kam.ReturnedColorSpace = 'grayscale';
triggerconfig(kam,'manual');
ImgSize = get(kam,'VideoResolution');
param = getselectedsource(kam);
param.ExposureMode = 'auto';
%param.ExposureMode = 'manual';
%param.Exposure = -7;

%% parameters
global AppRunning;
AppRunning  = 1;
img_Moravec = zeros(ImgSize(2),ImgSize(1),'uint8'); % Moravec array
CntCorners  = 10; % number of the most significant corners to display
CornerInd   = zeros(CntCorners,2,'uint16'); % corners coordinates
d = 10; % zero padding neighbourhood
k = 10; % size of a corner-cross in pixels
n = 0;  % image counter
TextColor = [1 1 0];
DrawColor = [0 1 0];
FontSize  = 16;

%% GUI
ScreenSize = get(0,'ScreenSize');
figure('Position',[10 1 ScreenSize(3)-20 ScreenSize(4)-70],'Color',[0.2 0.2 0.2],'KeyPressFcn',@MyKeyPressed);
hold on;

%% image processing main loop
start(kam);
tic;
while (AppRunning == 1)
    %% image counter
    n = n + 1;
    
    %% image acquisition
    img = getsnapshot(kam);
    
    %% Moravec operator
    for y = 2:ImgSize(2)-1
        for x = 2:ImgSize(1)-1
            sum = int16(0);
             center_value = int16(img(y,x)); % try int32
             for i = -1:+1
                 for j = -1:+1
                     sum = sum + abs(center_value-int16(img(y+i,x+j)));
                 end;
             end;
            img_Moravec(y,x) = uint8(sum/8);
        end;
    end;
    %% maximal responses
    for m = 1:CntCorners
        % maximum location
        MaxValue = max(max(img_Moravec));
        [ind_y, ind_x] = find(img_Moravec == MaxValue);
        ind_y = ind_y(1);
        ind_x = ind_x(1);
        % corners coordinates
        CornerInd(m,1) = ind_x;
        CornerInd(m,2) = ind_y;
        % zero padding
        y1 = ind_y-d;
        y2 = ind_y+d;
        x1 = ind_x-d;
        x2 = ind_x+d;
        if (y1<=0), y1=1; end;
        if (y2<=0), y2=1; end;
        if (x1<=0), x1=1; end;
        if (x2<=0), x2=1; end;
        img_Moravec(y1:y2,x1:x2) = 0;
    end;
    
    %% results
    subplot(2,2,1), imshow(img), title('Original image','Color',TextColor,'FontSize',FontSize);
    subplot(2,2,2), imshow(img_Moravec,[]), title(['Moravec operator, FPS: ',num2str(n/toc,'%.1f')],'Color',TextColor,'FontSize',FontSize);
    subplot(2,2,4), imshow(img), title(['Top ',num2str(CntCorners),' corners'],'Color',TextColor,'FontSize',FontSize);
    for m = 1:CntCorners
        if (m == 1) % the most significant corner
            DrawColorCorner = [1 0 0];
        else        % other corners
            DrawColorCorner = DrawColor;
        end;
        line([CornerInd(m,1)-k CornerInd(m,1)+k],[CornerInd(m,2) CornerInd(m,2)],'Color',DrawColorCorner);
        line([CornerInd(m,1) CornerInd(m,1)],[CornerInd(m,2)-k CornerInd(m,2)+k],'Color',DrawColorCorner);
    end;
    
    pause(0.1);
end;
stop(kam);
close all;


function MyKeyPressed(src, event) %#ok<INUSL>
%% key press
global AppRunning;
switch event.Key
    case 'escape'
        AppRunning = 0;
end;
