function DemonstrationSobel()
%*************************************************************************
%*                                                                       *
%*  Demonstration file of the Sobel operator for an edge detection. At   *
%*  first there is necessary to set up parameters of a camera in your    *
%*  computer in section "camera setting" below.                          *
%*                                                                       *
%*  Author: Karel Horak, Brno University of Technology, 2012             *
%*                                                                       *
%*************************************************************************

%% cleaning
clear all;
close all;
clc;

%% camera settings
imaqreset;
pause(2);
kam = videoinput('winvideo',1);
kam.ReturnedColorSpace = 'grayscale';
triggerconfig(kam,'manual');
param = getselectedsource(kam);
param.ExposureMode = 'auto';
%param.ExposureMode = 'manual';
%param.Exposure = -7;

%% parameters
global AppRunning;
AppRunning = 1;
n = 0;  % image counter
TextColor = [1 1 0];
FontSize  = 16;
h = fspecial('sobel');

%% GUI
ScreenSize = get(0,'ScreenSize');
figure('Position',[10 1 ScreenSize(3)-20 ScreenSize(4)-70],'Color',[0.2 0.2 0.2],'KeyPressFcn',@MyKeyPressed);
hold on;

%% image processing main loop
start(kam);
tic;
while (AppRunning == 1)
    %% image counter
    n = n + 1;
    
    %% image acquisition
    img = getsnapshot(kam);
    
    %% edge detection by means of the Sobel operator
    img_SobelH = imfilter(img, h, 'same');
    img_SobelV = imfilter(img, h', 'same');
    img_Sobel = img_SobelH/2 + img_SobelV/2;
    
    %% results
    subplot(2,2,1), imshow(img), title('Original image','Color',TextColor,'FontSize',FontSize);
    subplot(2,2,2), imshow(img_Sobel), title(['Sobel operator - both, FPS: ',num2str(n/toc,'%.1f')],'Color',TextColor,'FontSize',FontSize);
    subplot(2,2,3), imshow(img_SobelH), title('Sobel operator - horizontal','Color',TextColor,'FontSize',FontSize);
    subplot(2,2,4), imshow(img_SobelV), title('Sobel operator - vertical','Color',TextColor,'FontSize',FontSize);
    
    pause(0.1);
end;
stop(kam);
close all;


function MyKeyPressed(src, event) %#ok<INUSL>
%% key press
global AppRunning;
switch event.Key
    case 'escape'
        AppRunning = 0;
end;
